@@ -7,21 +7,25 @@ class TestConnectionPool < Minitest::Test
   end
 
   class NetworkConnection
-    SLEEP_TIME = 0.1
+    SLEEP_TIME = 0.02
 
     def initialize
       @x = 0
     end
 
+    def pass
+      Thread.pass
+    end
+
     def do_something(*_args, increment: 1)
       @x += increment
-      sleep SLEEP_TIME
+      sleep 0.02
       @x
     end
 
     def do_something_with_positional_hash(options)
       @x += options[:increment] || 1
-      sleep SLEEP_TIME
+      pass
       @x
     end
 
@@ -31,7 +35,7 @@ class TestConnectionPool < Minitest::Test
 
     def do_something_with_block
       @x += yield
-      sleep SLEEP_TIME
+      pass
       @x
     end
 
@@ -63,18 +67,13 @@ class TestConnectionPool < Minitest::Test
   end
 
   def kill_threads(threads)
-    threads.each do |thread|
-      thread.kill
-      thread.join
-    end
+    threads.each(&:kill)
+    threads.each(&:join)
   end
 
   def test_basic_multithreaded_usage
     pool_size = 5
     pool = ConnectionPool.new(size: pool_size) { NetworkConnection.new }
-
-    start = Time.new
-
     generations = 3
 
     result = Array.new(pool_size * generations) {
@@ -85,11 +84,7 @@ class TestConnectionPool < Minitest::Test
       end
     }.map(&:value)
 
-    finish = Time.new
-
     assert_equal((1..generations).cycle(pool_size).sort, result.sort)
-
-    assert_operator(finish - start, :>, generations * NetworkConnection::SLEEP_TIME)
   end
 
   def test_timeout
@@ -146,6 +141,14 @@ class TestConnectionPool < Minitest::Test
       end
     end
     assert_equal 1, pool.available
+    # Timeout.timeout creates a watcher thread and does not provide a way to
+    # shut it down so we have to disable maxitest's extra thread paranoia or
+    # else it will trigger a test failure.
+    skip_maxitest_extra_threads
+  end
+
+  def skip_maxitest_extra_threads
+    @maxitest_threads_before = Thread.list
   end
 
   def test_invalid_size
@@ -159,7 +162,7 @@ class TestConnectionPool < Minitest::Test
 
   def test_handle_interrupt_ensures_checkin
     pool = ConnectionPool.new(timeout: 0, size: 1) { Object.new }
-    def pool.checkout(options)
+    def pool.checkout(**options)
       sleep 0.015
       super
     end
@@ -226,7 +229,7 @@ class TestConnectionPool < Minitest::Test
 
     options = {foo: 123}
     e = assert_raises do
-      pool.with(options) {}
+      pool.with(**options) {}
     end
 
     assert_equal e.message, options.to_s
@@ -388,7 +391,7 @@ class TestConnectionPool < Minitest::Test
       pool.checkout
     end
 
-    assert pool.checkout timeout: 2 * NetworkConnection::SLEEP_TIME
+    assert pool.checkout(timeout: 2 * NetworkConnection::SLEEP_TIME)
   end
 
   def test_passthru
@@ -467,7 +470,7 @@ class TestConnectionPool < Minitest::Test
 
   def test_nested_discard
     recorder = Recorder.new
-    pool = ConnectionPool.new(size: 1) { {recorder: recorder} }
+    pool = ConnectionPool.new(size: 1, timeout: 0.01) { {recorder: recorder} }
     pool.with do |r_outer|
       @other = Thread.new { |t|
         pool.with do |r_other|
@@ -507,11 +510,11 @@ class TestConnectionPool < Minitest::Test
 
     kill_threads(threads)
 
-    assert_equal [["shutdown"]] * 3, recorders.map { |r| r.calls }
+    assert_equal %w[shutdown shutdown shutdown], recorders.map { |r| r.calls }.flatten
   end
 
   def test_checkout_after_reload_cannot_create_new_connections_beyond_size
-    pool = ConnectionPool.new(size: 1) { Object.new }
+    pool = ConnectionPool.new(size: 1, name: "bob") { Object.new }
     threads = use_pool pool, 1
     pool.reload {}
     assert_raises ConnectionPool::TimeoutError do
@@ -591,7 +594,7 @@ class TestConnectionPool < Minitest::Test
 
     assert_equal 1, pool.idle
 
-    pool.reap(0) { |recorder| recorder.do_work("reap") }
+    pool.reap(idle_seconds: 0) { |recorder| recorder.do_work("reap") }
 
     assert_equal 0, pool.idle
     assert_equal [["reap"]], recorders.map(&:calls)
@@ -607,7 +610,7 @@ class TestConnectionPool < Minitest::Test
 
     assert_equal 3, pool.idle
 
-    pool.reap(0) { |recorder| recorder.do_work("reap") }
+    pool.reap(idle_seconds: 0) { |recorder| recorder.do_work("reap") }
 
     assert_equal 0, pool.idle
     assert_equal [["reap"]] * 3, recorders.map(&:calls)
@@ -618,7 +621,7 @@ class TestConnectionPool < Minitest::Test
 
     pool.with { |conn| conn }
 
-    pool.reap(1000) { |conn| flunk "should not reap active connection" }
+    pool.reap(idle_seconds: 1000) { |conn| flunk "should not reap active connection" }
   end
 
   def test_idle_returns_number_of_idle_connections
@@ -655,7 +658,7 @@ class TestConnectionPool < Minitest::Test
     pool.shutdown {}
 
     assert_raises ConnectionPool::PoolShuttingDownError do
-      pool.reap(0) {}
+      pool.reap(idle_seconds: 0) {}
     end
   end
 
@@ -687,6 +690,9 @@ class TestConnectionPool < Minitest::Test
         assert_raises Timeout::Error do
           wrapper.with { flunk "connection checked out :(" }
         end
+        assert_raises Timeout::Error do
+          wrapper.with(timeout: 0.1) { flunk "connection checked out :(" }
+        end
       }.join
     end
 
@@ -805,7 +811,8 @@ class TestConnectionPool < Minitest::Test
         pool.with { |y| checkedout = y }
         checkedout
       end
-      result = r.take
+
+      result = (RUBY_VERSION < "4") ? r.take : r.value
       assert_equal obj, result # same string but different String instance
       refute_equal obj.object_id, result.object_id # was copied across Ractor boundary
     end
